@@ -5,9 +5,9 @@
 ###
 ### Helper functions for neighbourhood weight matrices in hhh4()
 ###
-### Copyright (C) 2012-2014 Sebastian Meyer
-### $Revision: 1025 $
-### $Date: 2014-09-22 17:58:48 +0200 (Mon, 22 Sep 2014) $
+### Copyright (C) 2012-2015 Sebastian Meyer
+### $Revision: 1367 $
+### $Date: 2015-06-09 14:59:24 +0200 (Tue, 09 Jun 2015) $
 ################################################################################
 
 
@@ -47,6 +47,89 @@ weightedSumNE <- function (observed, weights, lag)
 }
 
 
+### normalize weight matrix such that each row sums to 1 (at each time point)
+
+normalizeW <- function (W)
+{
+    dimW <- dim(W)
+    if (length(dimW) == 2L) {
+        W / .rowSums(W, dimW[1L], dimW[2L])
+    } else { # time-varying weights
+        res <- apply(W, 3L, normalizeW)
+        dim(res) <- dimW
+        res
+    }
+}
+
+
+### scale and/or normalize a weight matrix/array
+
+scaleNEweights.default <- function (weights, scale = NULL, normalize = FALSE)
+{
+    if (!is.null(scale))
+        weights <- scale * weights
+    if (normalize)
+        weights <- normalizeW(weights)
+    weights
+}
+
+scaleNEweights.list <- function (weights, scale = NULL, normalize = FALSE)
+{
+    if (is.null(scale) && !normalize)
+        return(weights)
+    
+    ## update parametric weights functions w, dw, d2w
+    if (normalize) {
+        dprod <- function (u, v, du, dv) du * v + u * dv
+        dfrac <- function (u, v, du, dv) (du * v - u * dv) / v^2
+        w <- function (...)
+            scaleNEweights.default(weights$w(...), scale, TRUE)
+        dw <- function (...) {
+            W <- scaleNEweights.default(weights$w(...), scale)
+            dW <- clapply(X = weights$dw(...), # matrix or list thereof
+                          FUN = scaleNEweights.default,
+                          scale = scale) # always returns a list
+            dimW <- dim(W)
+            normW <- .rowSums(W, dimW[1L], dimW[2L])
+            normdW <- lapply(X = dW, FUN = .rowSums, m = dimW[1L], n = dimW[2L])
+            mapply(FUN = dfrac, du = dW, dv = normdW,
+                   MoreArgs = list(u = W, v = normW),
+                   SIMPLIFY = FALSE, USE.NAMES = FALSE)
+        }
+        ## for d2w() we need all the stuff from dw() -> substitute
+        d2w <- as.function(c(alist(...=), substitute({
+            dWnorm <- DWBODY
+            d2W <- clapply(X = weights$d2w(...), # matrix or list thereof
+                           FUN = scaleNEweights.default,
+                           scale = scale) # always returns a list
+            normd2W <- lapply(X = d2W, FUN = .rowSums, m = dimW[1L], n = dimW[2L])
+            ## order of d2w is upper triangle BY ROW
+            dimd <- length(dW)
+            ri <- rep.int(seq_len(dimd), rep.int(dimd, dimd)) # row index
+            ci <- rep.int(seq_len(dimd), dimd)                # column index
+            uppertri <- ci >= ri
+            mapply(FUN = function (k, l, d2W, normd2W) {
+                       dfrac(dW[[k]], normW, d2W, normdW[[l]]) -
+                           dprod(W/normW, normdW[[k]]/normW, dWnorm[[l]],
+                                 dfrac(normdW[[k]], normW, normd2W, normdW[[l]]))
+                   },
+                   k = ri[uppertri], l = ci[uppertri],
+                   d2W = d2W, normd2W = normd2W,
+                   SIMPLIFY = FALSE, USE.NAMES = FALSE)
+        }, list(DWBODY = body(dw)))))
+    } else {
+        w <- function (...)
+            scaleNEweights.default(weights$w(...), scale)
+        dw <- function (...)
+            scaleNEweights.default(weights$dw(...), scale)
+        d2w <- function (...)
+            scaleNEweights.default(weights$d2w(...), scale)
+    }
+    
+    ## return list of functions
+    list(w = w, dw = dw, d2w = d2w, initial = weights$initial)
+}
+
 
 ##################################
 ### check ne$weights specification
@@ -55,19 +138,25 @@ weightedSumNE <- function (observed, weights, lag)
 
 ### checks for a fixed matrix/array
 
-checkWeightsArray <- function (W, nUnits, nTime, name)
+checkWeightsArray <- function (W, nUnits, nTime, name = deparse(substitute(W)),
+                               check0diag = FALSE, islands = FALSE)
 {
-    if (!is.array(W))
-        stop("'", name, "' must return a matrix/array")
+    if (!is.array(W) || !(length(dim(W)) %in% 2:3))
+        stop("'", name, "' must return a matrix or 3-dim array")
     if (any(dim(W)[1:2] != nUnits) || isTRUE(dim(W)[3] != nTime))
         stop("'", name, "' must conform to dimensions ",
              nUnits, " x ", nUnits, " (x ", nTime, ")")
-    if (any(is.na(W)))
+    if (any(is.na(W))) {
+        if (islands) # normalization of parametric weights yields division by 0
+            warning("neighbourhood structure contains islands")
         stop("missing values in '", name, "' are not allowed")
-    diags <- if (is.matrix(W)) diag(W) else apply(W, 3, diag)
-    if (any(diags != 0))
-        warning("'", name, "' has nonzeros on the diagonal",
-                if (!is.matrix(W)) "s")
+    }
+    if (check0diag) {
+        diags <- if (is.matrix(W)) diag(W) else apply(W, 3, diag)
+        if (any(diags != 0))
+            warning("'", name, "' has nonzeros on the diagonal",
+                    if (!is.matrix(W)) "s")
+    }
 }
 
 
@@ -95,9 +184,10 @@ checkWeightsFUN <- function (object)
 ### entry function for checks in hhh4()
 
 checkWeights <- function (weights, nUnits, nTime,
-                          nbmat, data)  # only used for parametric weights
+                          nbmat, data,  # only used for parametric weights
+                          check0diag = FALSE)
 {
-    name <- "control$ne$weights"
+    name <- deparse(substitute(weights)) # "control$ne$weights"
 
     ## check specification
     testweights <- if (is.array(weights)) weights else {
@@ -115,7 +205,11 @@ checkWeights <- function (weights, nUnits, nTime,
     
     ## apply matrix/array checks
     if (is.list(weights)) { # parametric weights
-        checkWeightsArray(testweights, nUnits, nTime, name=paste0(name, "$w"))
+        if (length(dim(testweights)) > 2L)
+            warning("time-varying parametric weights are not fully supported")
+        checkWeightsArray(testweights, nUnits, nTime, name = paste0(name, "$w"),
+                          check0diag = check0diag,
+                          islands = any(.rowSums(nbmat, nUnits, nUnits) == 0))
         dim.d <- length(weights$initial)
         dw <- weights$dw(weights$initial, nbmat, data)
         d2w <- weights$d2w(weights$initial, nbmat, data)
@@ -134,7 +228,8 @@ checkWeights <- function (weights, nUnits, nTime,
             lapply(d2w, checkWeightsArray, nUnits, nTime,
                    name=paste0(name, "$d2w[[i]]"))
         }
-    } else checkWeightsArray(testweights, nUnits, nTime, name=name)
+    } else checkWeightsArray(testweights, nUnits, nTime, name = name,
+                             check0diag = check0diag)
     
     ## Done
     invisible(TRUE)
@@ -149,17 +244,19 @@ checkWeights <- function (weights, nUnits, nTime,
 
 ### extract the (final) weight matrix/array from a fitted hhh4 object
 
-getNEweights <- function (object, pars = coefW(object))
+getNEweights <- function (object, pars = coefW(object),
+                          scale = ne$scale, normalize = ne$normalize)
 {
-    neweights <- object$control$ne$weights
-    
-    if (!is.list(neweights))  # NULL or fixed weight structure
-        return(neweights)
-
-    ## parametric weights
-    nd <- length(neweights$initial)
-    if (length(pars) != nd) stop("'pars' must be of length ", nd)
-    neweights$w(pars, neighbourhood(object$stsObj), object$control$data)
+    ne <- object$control$ne
+    weights <- if (is.list(ne$weights)) { # parametric weights
+        nd <- length(ne$weights$initial)
+        if (length(pars) != nd) stop("'pars' must be of length ", nd)
+        ne$weights$w(pars, neighbourhood(object$stsObj), object$control$data)
+    } else { # NULL or fixed weight structure
+        ne$weights
+    }
+    if (is.null(normalize)) normalize <- FALSE  # backward compatibility < 1.9-0
+    scaleNEweights.default(weights, scale, normalize)
 }
 
 
@@ -168,5 +265,7 @@ getNEweights <- function (object, pars = coefW(object))
 coefW <- function (object)
 {
     coefs <- if (inherits(object, c("hhh4","ah4"))) object$coefficients else object
-    coefs[grep("^neweights", names(coefs))]
+    coefW <- coefs[grep("^neweights", names(coefs))]
+    names(coefW) <- sub("^neweights\\.", "", names(coefW))
+    coefW
 }
