@@ -5,9 +5,9 @@
 ###
 ### Standard methods for hhh4-fits
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2014 Sebastian Meyer
-### $Revision: 1146 $
-### $Date: 2014-12-17 20:49:46 +0100 (Wed, 17 Dec 2014) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2015 Sebastian Meyer
+### $Revision: 1369 $
+### $Date: 2015-06-09 17:51:04 +0200 (Tue, 09 Jun 2015) $
 ################################################################################
 
 
@@ -353,6 +353,11 @@ update.hhh4 <- function (object, ..., S = NULL, subset.upper = NULL,
             SIMPLIFY = FALSE, USE.NAMES = FALSE
         )
     }
+    ## update initial values of parametric weight function
+    if (use.estimates && length(coefW <- coefW(object)) &&
+        ! "weights" %in% names(extras$ne)) { # only if function is unchanged
+        control$ne$weights$initial <- coefW
+    }
 
     ## adjust seasonality
     if (!is.null(S)) {
@@ -485,26 +490,36 @@ decompose.hhh4 <- function (x, coefs = x$coefficients, ...)
     res
 }
 
-## get the w_{ji} Y_{j,t-1} values from a fitted hhh4 model
+## get the w_{ji} Y_{j,t-1} values from a hhh4() fit
 ## (i.e., before summing the neighbourhood component over j)
 ## in an array with dimensions (t, i, j)
-neOffsetArray <- function (object, coefW = NULL, subset = object$control$subset)
+neOffsetArray <- function (object, pars = coefW(object),
+                           subset = object$control$subset)
 {
-    env <- new.env(hash = FALSE, parent = environment(terms(object)$offset$ne))
-    env$pars <- if (is.null(coefW)) coefW(object) else coefW
-    env$subset <- subset
-    res <- with(env, apply(neweights$w(pars, nbmat, data), 2L, function (wi) {
-        tm1 <- subset - lag
-        is.na(tm1) <- tm1 <= 0
-        t(Y[tm1,,drop=FALSE]) * wi
-    }))
-    dim(res) <- c(object$nUnit, length(subset), object$nUnit)
-    dimnames(res) <- list("j" = colnames(object$stsObj),
-                          "t" = rownames(object$stsObj)[subset],
-                          "i" = colnames(object$stsObj))
-    stopifnot(all.equal(colSums(res),  # sum over j
-                        terms(object)$offset$ne(env$pars)[subset,,drop=FALSE],
-                        check.attributes = FALSE))
+    ## initialize array ordered as (j, t, i) for apply() below
+    res <- array(data = 0,
+                 dim = c(object$nUnit, length(subset), object$nUnit),
+                 dimnames = list(
+                     "j" = colnames(object$stsObj),
+                     "t" = rownames(object$stsObj)[subset],
+                     "i" = colnames(object$stsObj)))
+    
+    ## calculate array values if the fit has an NE component
+    if ("ne" %in% componentsHHH4(object)) {
+        W <- getNEweights(object, pars = pars)
+        Y <- observed(object$stsObj)
+        nelag <- object$control$ne$lag
+        res[] <- apply(W, 2L, function (wi) {
+            tm1 <- subset - nelag
+            is.na(tm1) <- tm1 <= 0
+            t(Y[tm1,,drop=FALSE]) * wi
+        })
+        ## consistency check
+        stopifnot(all.equal(colSums(res),  # sum over j
+                            terms(object)$offset$ne(pars)[subset,,drop=FALSE],
+                            check.attributes = FALSE))
+    }
+    
     ## permute dimensions as (t, i, j)
     aperm(res, perm = c(2L, 3L, 1L), resize = TRUE)
 }
