@@ -62,11 +62,12 @@ clapply <- function (X, FUN, ...)
 
 ### pretty p-value formatting
 
-formatPval <- function (pv, eps = 1e-4)
+formatPval <- function (pv, eps = 1e-4, scientific = FALSE, ...)
 {
     format1 <- function (p)
-        format.pval(p, digits = if (p<10*eps) 1 else 2, eps = eps)
-    sapply(pv, format1)
+        format.pval(p, digits = if (p < 10*eps) 1 else 2, eps = eps,
+                    scientific = scientific, ...)
+    vapply(X = pv, FUN = format1, FUN.VALUE = "", USE.NAMES = TRUE)
 }
 
 
